#version 430

layout(location = 0) out vec4 color;
layout(location = 1) out vec4 BloomColor;

in vec3 Normal;
in vec3 FragPos;
in vec2 texCoord;
in vec4 fragPosLight;

uniform vec3 viewPos;

//ka, kd, ks, alpha
uniform struct Material
{
vec3 ambient;
vec3 diffuse;
vec3 specular;
float shininess;
} material;

uniform struct DirectionalLight{
vec3 dir;
vec3 color;
} dirL;

uniform struct PointLight
{
vec3 pos;
vec3 color;
vec3 attenuation;
} pointL;



uniform sampler2D texture_diffuse1;
layout(binding = 31) uniform sampler2D shadowMap;

vec3 calcDirLight(vec3 dir, vec3 normal, vec3 viewDir);
vec3 calcPointLight(vec3 pos, vec3 color, vec3 attenuation, vec3 normal, vec3 fragPos, vec3 viewDir);
float ShadowCalculation(vec4 fragPosLightSpace, float dotLightNormal);

const float levels = 3;

void main()
{
	
	//properties
	vec3 norm = normalize(Normal);
	vec3 viewDir = normalize(viewPos - FragPos);

	//add directional lighting
	vec3 result = calcDirLight(dirL.dir, norm, viewDir);
	color = vec4(result, 1.0);
	
	//add point light
	result += calcPointLight(pointL.pos, pointL.color, pointL.attenuation, norm, FragPos, viewDir);
	
	color = vec4(result, 1.0);

	//color = vec4(color * brightness2 * brightness2 * brightness2, 1.0);
	float brightness = (color.r * 0.2126) + (color.g * 0.7152) + (color.b * 0.0722); // Check with 0.5

	if (brightness > 0.9) {
		BloomColor = vec4(color.rgb, 1.0);
	}
	else {
		BloomColor = vec4(vec3(0.0), 1.0);
	}

}

vec3 calcDirLight(vec3 dir, vec3 normal, vec3 viewDir)
{	

	// ambient
	vec3 ambient = dirL.color * texture(texture_diffuse1, texCoord).rgb;

	// diffuse 
	vec3 lightDir = normalize(-dir);
	float brightness = max(dot(normal, lightDir), 0.0);
	float level = floor(brightness * levels);
	brightness = level / levels;
	vec3 diffuse = brightness * texture(texture_diffuse1, texCoord).rgb;

	// specular
	vec3 reflectDir = reflect(-lightDir, normal);
	float dampedFactor = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
	level = floor(dampedFactor * levels);
	dampedFactor = level / levels;
	vec3 specular = dampedFactor * texture(texture_diffuse1, texCoord).rgb;
	
	float shadow = ShadowCalculation(fragPosLight, dot(normal, lightDir));

	return (material.ambient * ambient * 0.1 + (material.diffuse * diffuse + material.specular * specular) * shadow);
}


vec3 calcPointLight(vec3 pos, vec3 color, vec3 attenuation, vec3 normal, vec3 fragPos, vec3 viewDir)
{

	// ambient
	vec3 ambient = color * texture(texture_diffuse1, texCoord).rgb;

	// diffuse
	vec3 lightDir = normalize(pos - fragPos);
	vec3 diffuse = max(dot(normal, lightDir), 0.0) * texture(texture_diffuse1, texCoord).rgb;

	// specular
	vec3 reflectDir = reflect(-lightDir, normal);
	vec3 specular = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess) * texture(texture_diffuse1, texCoord).rgb;

	// attenuation
	float distance = length(pos - fragPos);
	float result = 1.0 / (attenuation.x + attenuation.y * distance + attenuation.z * (distance * distance));

	ambient *= result;
	diffuse *= result;
	specular *= result;

	return (material.ambient * ambient + material.diffuse * diffuse + material.specular * specular);
}

float ShadowCalculation(vec4 fragPosLightSpace, float dotLightNormal)
{
	// transform to range [0,1]
	vec3 pos = fragPosLightSpace.xyz * 0.5 + 0.5;
	if (pos.z > 1.0) {
		pos.z = 1.0;
	}

	float bias = max(0.01 * (1.0 - dotLightNormal), 0.0005);

	// PCF
	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for (int x = -1; x <= 1; x++) 
	{
		for (int y = -1; y <= 1; y++) 
		{
			float depth = texture(shadowMap, pos.xy + vec2(x,y) * texelSize).r;
			shadow += (depth + bias) < pos.z ? 0.0 : 1.0;
		}
	}

	return shadow / 9.0;
}